/*
 Copyright (c) 2012-2018 Open Lab
 Written by Roberto Bicchierai and Silvia Chelazzi http://roberto.open-lab.com
 Permission is hereby granted, free of charge, to any person obtaining
 a copy of this software and associated documentation files (the
 "Software"), to deal in the Software without restriction, including
 without limitation the rights to use, copy, modify, merge, publish,
 distribute, sublicense, and/or sell copies of the Software, and to
 permit persons to whom the Software is furnished to do so, subject to
 the following conditions:

 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*/
Ganttalendar.prototype.initZoomlevels = function (master) {
    //console.debug("Ganttalendar.prototype.initZoomlevels");

    var self = this;
    /* <Modify the source code> */
    /*
     * To configure multiple languages
     */
    this.master = master;
    /* </Modify the source code> */

    // define the zoom level arrays 
    this.zoomLevels = [];
    this.zoomDrawers = {};


    function _addZoom(zoom, zoomDrawer) {
        self.zoomLevels.push(zoom);
        self.zoomDrawers[zoom] = zoomDrawer;

        //compute the scale
        // TODO: window zoom to the type of week, svg starts x value calculation will be later, if 600 -> 1080, sometimes solve the problem， not repaired temporarily
        self.zoomDrawers[zoom].computedScaleX = 600 / millisFromString(zoom);
    }


    //-----------------------------  3 DAYS  600px-----------------------------
    _addZoom("3d", {
        adjustDates: function (start, end) {
            /* <Modify the source code> */
            /*
             * <Origin Code>
             * start.setFirstDayOfThisWeek();
             * end.setFirstDayOfThisWeek();
             * </Origin Code>
             */
            start.setDate(start.getDate() - 3);
            end.setDate(end.getDate() + 6);
            /* </Modify the source code> */
        },
        row1: function (date, ctxHead) {
            var start = new Date(date.getTime());
            date.setDate(date.getDate() + 6);
            /* <Modify the source code> */
            /*
             * <Origin Code>
             * self.createHeadCell(1, this, ctxHead, start.format("MMMM d") + " - " + date.format("MMMM d yyyy") + " (" + start.format("w") + ")", 7, "", start, date);
             * </Origin Code>
             */
            var langParams = {
                startMonth: self.master.lang.Zoom.MonthAbbreviationsNumber[start.getMonth()],
                startDate: start.format("d"),
                dateYear: date.format("yyyy"),
                dateMonth: self.master.lang.Zoom.MonthAbbreviationsNumber[start.getMonth()],
                dateDay: date.format("d"),
                whickWeek: self.master.formatLang(self.master.lang.Zoom.WhickWeek, date.format("w")),
            }
            self.createHeadCell(1, this, ctxHead, self.master.formatLang(self.master.lang.Zoom.ThreeDays.Row1, langParams), 7, "", start, date);
            /* </Modify the source code> */
            date.setDate(date.getDate() + 1);
        },
        row2: function (date, ctxHead, ctxBody) {
            var start = new Date(date.getTime());
            date.setDate(date.getDate() + 1);
            var holyClass = isHoliday(start) ? "holy" : "";
            /* <Modify the source code> */
            /*
             * <Origin Code>
             * self.createHeadCell(2, this, ctxHead, start.format("EEE d"), 1, "pluginGantt_headSmall " + holyClass, start, date);
             * </Origin Code>
             */
            var langParams = {
                startDay: self.master.lang.Zoom.DayNamesNumber[start.getDay()],
                startDate: start.format("dd")
            }
            self.createHeadCell(2, this, ctxHead, self.master.formatLang(self.master.lang.Zoom.ThreeDays.Row2, langParams), 1, "pluginGantt_headSmall " + holyClass, start, date);
            /* </Modify the source code> */
            self.createBodyCell(this, ctxBody, 1, start.getDay() % 7 == (self.master.firstDayOfWeek + 6) % 7, holyClass);
        }
    });



    //-----------------------------  1 WEEK  600px -----------------------------
    _addZoom("1w", {
        adjustDates: function (start, end) {
            //reset day of week
            start.setFirstDayOfThisWeek();
            start.setDate(start.getDate() - 7);
            end.setFirstDayOfThisWeek();
            /* <Modify the source code> */
            /*
             * <Origin Code>
             * end.setDate(end.getDate() + 13);
             * </Origin Code>
             */
            end.setDate(end.getDate() + 20);
            /* </Modify the source code> */
        },
        row1: function (date, ctxHead) {
            var start = new Date(date.getTime());
            date.setDate(date.getDate() + 6);
            /* <Modify the source code> */
            /*
             * <Origin Code>
             * self.createHeadCell(1, this, ctxHead, start.format("MMM d") + " - " + date.format("MMM d 'yy") + " (" + self.master.lang.Error["GANTT_WEEK_SHORT"] + date.format("w") + ")", 7, "", start, date);
             * </Origin Code>
             */
            var langParams = {
                startMonth: self.master.lang.Zoom.MonthAbbreviationsNumber[start.getMonth()],
                startDate: start.format("d"),
                dateYear: date.format("yyyy"),
                dateMonth: self.master.lang.Zoom.MonthAbbreviationsNumber[start.getMonth()],
                dateDay: date.format("d"),
                whickWeek: self.master.formatLang(self.master.lang.Zoom.WhickWeek, date.format("w")),
            }
            self.createHeadCell(1, this, ctxHead, self.master.formatLang(self.master.lang.Zoom.OneWeek.Row1, langParams), 7, "", start, date);
            /* </Modify the source code> */
            date.setDate(date.getDate() + 1);
        },
        row2: function (date, ctxHead, ctxBody) {
            var start = new Date(date.getTime());
            date.setDate(date.getDate() + 1);
            var holyClass = isHoliday(start) ? "holy" : "";
            /* <Modify the source code> */
            /*
             * <Origin Code>
             * self.createHeadCell(2, this, ctxHead, start.format("EEEE").substr(0, 1) + " (" + start.format("dd") + ")", 1, "pluginGantt_headSmall " + holyClass, start, date);
             * </Origin Code>
             */
            var langParams = {
                startDay: self.master.lang.Zoom.DayAbbreviationsNumber[start.getDay()],
                startDate: start.format("dd")
            }
            self.createHeadCell(2, this, ctxHead, self.master.formatLang(self.master.lang.Zoom.OneWeek.Row2, langParams), 1, "pluginGantt_headSmall " + holyClass, start, date);
            /* </Modify the source code> */
            self.createBodyCell(this, ctxBody, 1, start.getDay() % 7 == (self.master.firstDayOfWeek + 6) % 7, holyClass);
        }
    });


    //-----------------------------  2 WEEKS  600px -----------------------------
    _addZoom("2w", {
        adjustDates: function (start, end) {
            start.setFirstDayOfThisWeek();
            start.setDate(start.getDate() - 7);
            end.setFirstDayOfThisWeek();
            end.setDate(end.getDate() + 20);
        },
        row1: function (date, tr1) {
            var start = new Date(date.getTime());
            date.setDate(date.getDate() + 6);
            /* <Modify the source code> */
            /*
             * <Origin Code>
             * self.createHeadCell(1, this, tr1, start.format("MMM d") + " - " + date.format("MMM d 'yy") + " (" + self.master.lang.Error["GANTT_WEEK_SHORT"] + date.format("w") + ")", 7, "", start, date);
             * </Origin Code>
             */
            var langParams = {
                startMonth: self.master.lang.Zoom.MonthAbbreviationsNumber[start.getMonth()],
                startDate: start.format("d"),
                dateYear: date.format("yyyy"),
                dateMonth: self.master.lang.Zoom.MonthAbbreviationsNumber[start.getMonth()],
                dateDay: date.format("d"),
                whickWeek: self.master.formatLang(self.master.lang.Zoom.WhickWeek, date.format("w")),
            }
            self.createHeadCell(1, this, tr1, self.master.formatLang(self.master.lang.Zoom.TwoWeek.Row1, langParams), 7, "", start, date);
            /* </Modify the source code> */
            date.setDate(date.getDate() + 1);
        },
        row2: function (date, tr2, trBody) {
            var start = new Date(date.getTime());
            date.setDate(date.getDate() + 1);
            var holyClass = isHoliday(start) ? "holy" : "";
            /* <Modify the source code> */
            /*
             * <Origin Code>
             * self.createHeadCell(2, this, tr2, start.format("EEEE").substr(0, 1), 1, "pluginGantt_headSmall " + holyClass, start, date);
             * </Origin Code>
             */
            var langParams = {
                startDay: self.master.lang.Zoom.DayAbbreviationsNumber[start.getDay()]
            }
            self.createHeadCell(2, this, tr2, self.master.formatLang(self.master.lang.Zoom.TwoWeek.Row2, langParams), 1, "pluginGantt_headSmall " + holyClass, start, date);
            /* </Modify the source code> */
            self.createBodyCell(this, trBody, 1, start.getDay() % 7 == (self.master.firstDayOfWeek + 6) % 7, holyClass);
        }
    });


    //-----------------------------  1 MONTH  600px  -----------------------------
    _addZoom("1M", {
        adjustDates: function (start, end) {
            /* <Modify the source code> */
            /*
             * <Origin Code>
             * start.setMonth(start.getMonth() - 1);
             * </Origin Code>
             */
            start.setMonth(start.getMonth() - 2);
            /* </Modify the source code> */
            start.setDate(15);
            end.setDate(1);
            /* <Modify the source code> */
            /*
             * <Origin Code>
             * end.setMonth(end.getMonth() + 1);
             * </Origin Code>
             */
            end.setMonth(end.getMonth() + 3);
            /* </Modify the source code> */
            end.setDate(end.getDate() + 14);
        },
        row1: function (date, tr1) {
            var start = new Date(date.getTime());
            date.setDate(1);
            date.setMonth(date.getMonth() + 1);
            date.setDate(date.getDate() - 1);
            var inc = date.getDate() - start.getDate() + 1;
            date.setDate(date.getDate() + 1);
            /* <Modify the source code> */
            /*
             * <Origin Code>
             * self.createHeadCell(1, this, tr1, start.format("MMMM yyyy"), inc, "", start, date); //spans mumber of dayn in the month
             * </Origin Code>
             */
            var langParams = {
                startYear: start.format("yyyy"),
                startMonth: start.format("MMMM"),
            }
            self.createHeadCell(1, this, tr1, self.master.formatLang(self.master.lang.Zoom.OneMonth.Row1, langParams), inc, "", start, date); //spans mumber of dayn in the month
            /* </Modify the source code> */
        },
        row2: function (date, tr2, trBody) {
            var start = new Date(date.getTime());
            date.setDate(date.getDate() + 1);
            var holyClass = isHoliday(start) ? "holy" : "";
            /* <Modify the source code> */
            /*
             * <Origin Code>
             * self.createHeadCell(2, this, tr2, start.format("d"), 1, "pluginGantt_headSmall " + holyClass, start, date);
             * </Origin Code>
             */
            var langParams = {
                startDate: start.format("d")
            };
            self.createHeadCell(2, this, tr2, self.master.formatLang(self.master.lang.Zoom.OneMonth.Row2, langParams), 1, "pluginGantt_headSmall " + holyClass, start, date);
            /* </Modify the source code> */
            var nd = new Date(start.getTime());
            nd.setDate(start.getDate() + 1);
            self.createBodyCell(this, trBody, 1, nd.getDate() == 1, holyClass);
        }
    });



    //-----------------------------  1 QUARTERS   -----------------------------
    _addZoom("1Q", {
        adjustDates: function (start, end) {
            start.setDate(1);
            /* <Modify the source code> */
            /*
             * <Origin Code>
             * start.setMonth(Math.floor(start.getMonth() / 3) * 3 - 1);
             * </Origin Code>
             */
            start.setMonth(Math.floor(start.getMonth() / 3) * 3 - 3);
            /* </Modify the source code> */
            end.setDate(1);
            /* <Modify the source code> */
            /*
             * <Origin Code>
             * end.setMonth(Math.floor(end.getMonth() / 3) * 3 + 4);
             * </Origin Code>
             */
            end.setMonth(Math.floor(end.getMonth() / 3) * 3 + 9);
            /* </Modify the source code> */
            end.setDate(end.getDate() - 1);
        },
        row1: function (date, tr1) {
            var start = new Date(date.getTime());
            date.setMonth(Math.floor(date.getMonth() / 3) * 3 + 3);
            /* <Modify the source code> */
            /*
             * bug: November and December will be the first quarter of next year.
             * inc === span(colSpan)
             * 
             * <Origin Code>
             * var inc = (date.getMonth() - start.getMonth());
             * inc = inc > 0 ? inc : 1;
             * var q = (Math.floor(start.getMonth() / 3) + 1);
             * self.createHeadCell(1, this, ctxHead, start.format("MMMM d") + " - " + date.format("MMMM d yyyy") + " (" + start.format("w") + ")", 7, "", start, date);
             * </Origin Code>
             */
            var span = (date.getMonth() - start.getMonth());
            span = span > 0 ? span : 3;
            var q = (Math.floor(start.getMonth() / 3) + 1);
            var langParams = {
                startYear: start.format("yyyy"),
                startQuarter: self.master.lang.Zoom.QuarterNumber[q - 1]
            }
            self.createHeadCell(1, this, tr1, self.master.formatLang(self.master.lang.Zoom.OneQuarter.Row1, langParams), span, "", start, date);
            /* </Modify the source code> */
        },
        row2: function (date, tr2, trBody) {
            var start = new Date(date.getTime());
            date.setMonth(date.getMonth() + 1);
            /* <Modify the source code> */
            /*
             * <Origin Code>
             * self.createHeadCell(2, this, tr2, start.format("MMMM"), 1, "pluginGantt_headSmall", start, date);
             * </Origin Code>
             */
            var langParams = {
                startMonth: start.format("MMMM")
            };
            self.createHeadCell(2, this, tr2, self.master.formatLang(self.master.lang.Zoom.OneQuarter.Row2, langParams), 1, "pluginGantt_headSmall", start, date);
            /* </Modify the source code> */
            self.createBodyCell(this, trBody, 1, start.getMonth() % 3 == 2);
        }
    });


    //-----------------------------  2 QUARTERS   -----------------------------
    _addZoom("2Q", {
        adjustDates: function (start, end) {
            start.setDate(1);
            /* <Modify the source code> */
            /*
             * <Origin Code>
             * start.setMonth(Math.floor(start.getMonth() / 3) * 3 - 3);
             * </Origin Code>
             */
            start.setMonth(Math.floor(start.getMonth() / 3) * 3 - 6);
            /* </Modify the source code> */
            end.setDate(1);
            /*
             * <Origin Code>
             * end.setMonth(Math.floor(end.getMonth() / 3) * 3 + 6);
             * </Origin Code>
             */
            end.setMonth(Math.floor(end.getMonth() / 3) * 3 + 15);
            /* </Modify the source code> */
            end.setDate(end.getDate() - 1);
        },
        row1: function (date, tr1) {
            var start = new Date(date.getTime());
            date.setMonth(date.getMonth() + 3);
            var q = (Math.floor(start.getMonth() / 3) + 1);
            /* <Modify the source code> */
            /*
             * <Origin Code>
             * self.createHeadCell(1, this, tr1, self.master.lang.Error["GANTT_QUARTER"] + " " + q + " " + start.format("yyyy"), 3, "", start, date);
             * </Origin Code>
             */
            var langParams = {
                startYear: start.format("yyyy"),
                startQuarter: self.master.lang.Zoom.QuarterNumber[q - 1]
            }
            self.createHeadCell(1, this, tr1, self.master.formatLang(self.master.lang.Zoom.TwoQuarter.Row1, langParams), 3, "", start, date);
            /* </Modify the source code> */
        },
        row2: function (date, tr2, trBody) {
            var start = new Date(date.getTime());
            date.setMonth(date.getMonth() + 1);
            /* <Modify the source code> */
            /*
             * <Origin Code>
             * var lbl = start.format("MMMM");
             * </Origin Code>
             */
            var langParams = {
                startMonth: start.format("MMMM")
            };
            var lbl = self.master.formatLang(self.master.lang.Zoom.TwoQuarter.Row2, langParams);
            /* </Modify the source code> */
            self.createHeadCell(2, this, tr2, lbl, 1, "pluginGantt_headSmall", start, date);
            self.createBodyCell(this, trBody, 1, start.getMonth() % 3 == 2);
        }
    });


    //-----------------------------  1 YEAR  -----------------------------
    _addZoom("1y", {
        adjustDates: function (start, end) {
            start.setDate(1);
            /*
             * <Origin Code>
             * start.setMonth(Math.floor(start.getMonth() / 6) * 6 - 6);
             * </Origin Code>
             */
            start.setMonth(Math.floor(start.getMonth() / 6) * 6 - 12);
            /* </Modify the source code> */
            end.setDate(1);
            /*
             * <Origin Code>
             * end.setMonth(Math.floor(end.getMonth() / 6) * 6 + 12);
             * </Origin Code>
             */
            end.setMonth(Math.floor(end.getMonth() / 6) * 6 + 30);
            /* </Modify the source code> */
            end.setDate(end.getDate() - 1);
        },
        row1: function (date, tr1) {
            var start = new Date(date.getTime());
            date.setMonth(date.getMonth() + 6);
            var sem = (Math.floor(start.getMonth() / 6) + 1);
            /* <Modify the source code> */
            /*
             * <Origin Code>
             * self.createHeadCell(1, this, tr1, self.master.lang.Error["GANTT_SEMESTER"] + " " + sem + "-" + start.format("yyyy"), 6, "", start, date);
             * </Origin Code>
             */
            var langParams = {
                startYear: start.format("yyyy"),
                HalfYear: self.master.lang.Zoom.HalfYearNumber[sem - 1]
            }
            self.createHeadCell(1, this, tr1, self.master.formatLang(self.master.lang.Zoom.OneYear.Row1, langParams), 6, "", start, date);
            /* </Modify the source code> */
        },
        row2: function (date, tr2, trBody) {
            var start = new Date(date.getTime());
            date.setMonth(date.getMonth() + 1);
            /* <Modify the source code> */
            /*
             * <Origin Code>
             * self.createHeadCell(2, this, tr2, start.format("MMM"), 1, "pluginGantt_headSmall", start, date);
             * </Origin Code>
             */
            var langParams = {
                startMonth: start.format("MMM")
            };
            self.createHeadCell(2, this, tr2, self.master.formatLang(self.master.lang.Zoom.OneYear.Row2, langParams), 1, "pluginGantt_headSmall", start, date);
            /* </Modify the source code> */
            self.createBodyCell(this, trBody, 1, (start.getMonth() + 1) % 6 == 0);
        }
    });


    //-----------------------------  2 YEAR -----------------------------
    _addZoom("2y", {
        adjustDates: function (start, end) {
            start.setDate(1);
            /*
             * <Origin Code>
             * start.setMonth(-6);
             * </Origin Code>
             */
            start.setMonth(-24);
            /* </Modify the source code> */
            end.setDate(30);
            /*
             * <Origin Code>
             * end.setMonth(17);
             * </Origin Code>
             */
            end.setMonth(59);
            /* </Modify the source code> */
        },
        row1: function (date, tr1) {
            var start = new Date(date.getTime());
            var inc = 12 - start.getMonth();
            date.setMonth(date.getMonth() + inc);
            /* <Modify the source code> */
            /*
             * <Origin Code>
             * self.createHeadCell(1, this, tr1, start.format("yyyy"), inc / 6, "", start, date);
             * </Origin Code>
             */
            var langParams = {
                startYear: start.format("yyyy")
            }
            self.createHeadCell(1, this, tr1, self.master.formatLang(self.master.lang.Zoom.TwoYear.Row1, langParams), inc / 6, "", start, date);
            /* </Modify the source code> */
        },
        row2: function (date, tr2, trBody) {
            var start = new Date(date.getTime());
            date.setMonth(date.getMonth() + 6);
            var sem = (Math.floor(start.getMonth() / 6) + 1);
            /* <Modify the source code> */
            /*
             * <Origin Code>
             * self.createHeadCell(2, this, tr2, self.master.lang.Error["GANTT_SEMESTER"] + " " + sem, 1, "pluginGantt_headSmall", start, date);
             * </Origin Code>
             */
            var langParams = {
                HalfYear: self.master.lang.Zoom.HalfYearNumber[sem - 1]
            };
            self.createHeadCell(2, this, tr2, self.master.formatLang(self.master.lang.Zoom.TwoYear.Row2, langParams), 1, "pluginGantt_headSmall", start, date);
            /* </Modify the source code> */
            self.createBodyCell(this, trBody, 1, sem == 2);
        }
    });



};

